try:
    from gpt4all import GPT4All
except (ImportError, ModuleNotFoundError):
    GPT4All = None

_loaded_models = {}


def get_chat_completion(messages, model="ggml-gpt4all-j-v1.3-groovy"):
    if GPT4All is None:
        raise ImportError(
            "gpt4all is not installed, try `pip install scikit-llm[gpt4all]`"
        )
    if model not in _loaded_models.keys():
        _loaded_models[model] = GPT4All(model)
    return _loaded_models[model].chat_completion(
        messages, verbose=False, streaming=False, temp=1e-10
    )


def unload_models():
    global _loaded_models
    _loaded_models = {}



from skllm.gpt4all_client import get_chat_completion as _g4a_get_chat_completion
from skllm.openai.chatgpt import get_chat_completion as _oai_get_chat_completion


def get_chat_completion(
    messages, openai_key=None, openai_org=None, model="gpt-3.5-turbo", max_retries=3
):
    if model.startswith("gpt4all::"):
        return _g4a_get_chat_completion(messages, model[9:])
    else:
        return _oai_get_chat_completion(
            messages, openai_key, openai_org, model, max_retries
        )



import unittest
from unittest.mock import patch
from skllm.openai.chatgpt import construct_message, get_chat_completion, extract_json_key

class TestChatGPT(unittest.TestCase):

    @patch("skllm.openai.credentials.set_credentials")
    @patch("openai.ChatCompletion.create")
    def test_get_chat_completion(self, mock_create, mock_set_credentials):
        messages = [{"role": "system", "content": "Hello"}]
        key = "some_key"
        org = "some_org"
        model = "gpt-3.5-turbo"
        mock_create.side_effect = [Exception("API error"), "success"]

        result = get_chat_completion(messages, key, org, model)

        self.assertTrue(mock_set_credentials.call_count <= 1, "set_credentials should be called at most once")
        self.assertEqual(mock_create.call_count, 2, "ChatCompletion.create should be called twice due to an exception on the first call")
        self.assertEqual(result, "success")

    def test_construct_message(self):
        role = "user"
        content = "Hello, World!"
        message = construct_message(role, content)
        self.assertEqual(message, {"role": role, "content": content})
        with self.assertRaises(ValueError):
            construct_message("invalid_role", content)

    def test_extract_json_key(self):
        json_ = '{"key": "value"}'
        key = "key"
        result = extract_json_key(json_, key)
        self.assertEqual(result, "value")

        # Given that the function returns None when a KeyError occurs, adjust the assertion
        result_with_invalid_key = extract_json_key(json_, "invalid_key")
        self.assertEqual(result_with_invalid_key, None)


if __name__ == '__main__':
    unittest.main()



ZERO_SHOT_CLF_PROMPT_TEMPLATE = """
You will be provided with the following information:
1. An arbitrary text sample. The sample is delimited with triple backticks.
2. List of categories the text sample can be assigned to. The list is delimited with square brackets. The categories in the list are enclosed in the single quotes and comma separated.

Perform the following tasks:
1. Identify to which category the provided text belongs to with the highest probability.
2. Assign the provided text to that category.
3. Provide your response in a JSON format containing a single key `label` and a value corresponding to the assigned category. Do not provide any additional information except the JSON.

List of categories: {labels}

Text sample: ```{x}```

Your JSON response:
"""

FEW_SHOT_CLF_PROMPT_TEMPLATE = """
You will be provided with the following information:
1. An arbitrary text sample. The sample is delimited with triple backticks.
2. List of categories the text sample can be assigned to. The list is delimited with square brackets. The categories in the list are enclosed in the single quotes and comma separated.
3. Examples of text samples and their assigned categories. The examples are delimited with triple backticks. The assigned categories are enclosed in a list-like structure. These examples are to be used as training data.

Perform the following tasks:
1. Identify to which category the provided text belongs to with the highest probability.
2. Assign the provided text to that category.
3. Provide your response in a JSON format containing a single key `label` and a value corresponding to the assigned category. Do not provide any additional information except the JSON.

List of categories: {labels}

Training data:
{training_data}

Text sample: ```{x}```

Your JSON response:
"""

ZERO_SHOT_MLCLF_PROMPT_TEMPLATE = """
You will be provided with the following information:
1. An arbitrary text sample. The sample is delimited with triple backticks.
2. List of categories the text sample can be assigned to. The list is delimited with square brackets. The categories in the list are enclosed in the single quotes and comma separated. The text sample belongs to at least one category but cannot exceed {max_cats}.

Perform the following tasks:
1. Identify to which categories the provided text belongs to with the highest probability.
2. Assign the text sample to at least 1 but up to {max_cats} categories based on the probabilities.
3. Provide your response in a JSON format containing a single key `label` and a value corresponding to the list of assigned categories. Do not provide any additional information except the JSON.

List of categories: {labels}

Text sample: ```{x}```

Your JSON response:
"""

SUMMARY_PROMPT_TEMPLATE = """
Your task is to generate a summary of the text sample.
Summarize the text sample provided below, delimited by triple backticks, in at most {max_words} words.

Text sample: ```{x}```
Summarized text:
"""

TRANSLATION_PROMPT_TEMPLATE = """
You will be provided with an arbitrary text sample, delimited by triple backticks.
Your task is to translate this text to {output_language} language and output the translated text.

Perform the following actions:
1. Determine the language of the text sample.
2. If the language is not {output_language}, translate the text sample to {output_language} language.
3. Output the translated text.
If the text sample provided is not in a recognizable language, output "No translation available".
Do not output any additional information except the translated text.

Text sample: ```{x}```
Translated text:
"""



from typing import List, Union

import numpy as np
import pandas as pd

from skllm.models.gpt_zero_shot_clf import (
    ZeroShotGPTClassifier as _ZeroShotGPTClassifier,
)
from skllm.prompts.builders import build_few_shot_prompt_slc
from skllm.utils import to_numpy as _to_numpy

_TRAINING_SAMPLE_PROMPT_TEMPLATE = """
Sample input:
```{x}```

Sample target: {label}
"""


class FewShotGPTClassifier(_ZeroShotGPTClassifier):
    """Few-shot single-label classifier."""

    def fit(
        self,
        X: Union[np.ndarray, pd.Series, List[str]],
        y: Union[np.ndarray, pd.Series, List[str]],
    ):
        """Fits the model by storing the training data and extracting the
        unique targets.

        Parameters
        ----------
        X : Union[np.ndarray, pd.Series, List[str]]
            training data
        y : Union[np.ndarray, pd.Series, List[str]]
            training labels

        Returns
        -------
        FewShotGPTClassifier
            self
        """
        if not len(X) == len(y):
            raise ValueError("X and y must have the same length.")
        X = _to_numpy(X)
        y = _to_numpy(y)
        self.training_data_ = (X, y)
        self.classes_, self.probabilities_ = self._get_unique_targets(y)
        return self

    def _get_prompt(self, x: str) -> str:
        """Generates the prompt for the given input.

        Parameters
        ----------
        x : str
            sample to classify

        Returns
        -------
        str
            final prompt
        """
        training_data = []
        for xt, yt in zip(*self.training_data_):
            training_data.append(
                _TRAINING_SAMPLE_PROMPT_TEMPLATE.format(x=xt, label=yt)
            )

        training_data_str = "\n".join(training_data)

        return build_few_shot_prompt_slc(
            x=x, training_data=training_data_str, labels=repr(self.classes_)
        )



import openai
from time import sleep
from skllm.openai.credentials import set_credentials

def get_embedding(
    text, key: str, org: str, model="text-embedding-ada-002", max_retries=3
):
    set_credentials(key, org)
    text = text.replace("\n", " ")
    error_msg = None
    error_type = None
    for _ in range(max_retries):
        try:
            emb = openai.Embedding.create(input=[text], model=model)["data"][0][
                "embedding"
            ]
            if not isinstance(emb, list):
                raise ValueError(f"Encountered unknown embedding format. Expected list, got {type(emb)}")
            return emb
        except Exception as e:
            error_msg = str(e)
            error_type =  type(e).__name__
            sleep(3)
    raise RuntimeError(
        f"Could not obtain the embedding after {max_retries} retries: `{error_type} :: {error_msg}`"
    )



import random
from abc import ABC, abstractmethod
from collections import Counter
from typing import Any, List, Optional, Union

import numpy as np
import pandas as pd
from sklearn.base import BaseEstimator, ClassifierMixin
from tqdm import tqdm

from skllm.completions import get_chat_completion
from skllm.openai.chatgpt import construct_message, extract_json_key
from skllm.openai.mixin import OpenAIMixin as _OAIMixin
from skllm.prompts.builders import (
    build_zero_shot_prompt_mlc,
    build_zero_shot_prompt_slc,
)
from skllm.utils import to_numpy as _to_numpy


class _BaseZeroShotGPTClassifier(ABC, BaseEstimator, ClassifierMixin, _OAIMixin):
    def __init__(
        self,
        openai_key: Optional[str] = None,
        openai_org: Optional[str] = None,
        openai_model: str = "gpt-3.5-turbo",
    ):
        self._set_keys(openai_key, openai_org)
        self.openai_model = openai_model

    def _to_np(self, X):
        return _to_numpy(X)

    def fit(
        self,
        X: Optional[Union[np.ndarray, pd.Series, List[str]]],
        y: Union[np.ndarray, pd.Series, List[str], List[List[str]]],
    ):
        X = self._to_np(X)
        self.classes_, self.probabilities_ = self._get_unique_targets(y)
        return self

    def predict(self, X: Union[np.ndarray, pd.Series, List[str]]):
        X = self._to_np(X)
        predictions = []
        for i in tqdm(range(len(X))):
            predictions.append(self._predict_single(X[i]))
        return predictions

    @abstractmethod
    def _extract_labels(self, y: Any) -> List[str]:
        pass

    def _get_unique_targets(self, y):
        labels = self._extract_labels(y)

        counts = Counter(labels)

        total = sum(counts.values())

        classes, probs = [], []
        for l, c in counts.items():
            classes.append(l)
            probs.append(c / total)

        return classes, probs

    def _get_chat_completion(self, x):
        prompt = self._get_prompt(x)
        msgs = []
        msgs.append(construct_message("system", "You are a text classification model."))
        msgs.append(construct_message("user", prompt))
        completion = get_chat_completion(
            msgs, self._get_openai_key(), self._get_openai_org(), self.openai_model
        )
        return completion


class ZeroShotGPTClassifier(_BaseZeroShotGPTClassifier):
    def __init__(
        self,
        openai_key: Optional[str] = None,
        openai_org: Optional[str] = None,
        openai_model: str = "gpt-3.5-turbo",
    ):
        super().__init__(openai_key, openai_org, openai_model)

    def _extract_labels(self, y: Any) -> List[str]:
        if isinstance(y, (pd.Series, np.ndarray)):
            labels = y.tolist()
        else:
            labels = y
        return labels

    def _get_prompt(self, x) -> str:
        return build_zero_shot_prompt_slc(x, repr(self.classes_))

    def _predict_single(self, x):
        completion = self._get_chat_completion(x)
        try:
            if self.openai_model.startswith("gpt4all::"):
                label = str(
                    extract_json_key(
                        completion["choices"][0]["message"]["content"], "label"
                    )
                )
            else:
                label = str(
                    extract_json_key(completion.choices[0].message["content"], "label")
                )
        except Exception as e:
            print(completion)
            print(f"Could not extract the label from the completion: {str(e)}")
            label = ""

        if label not in self.classes_:
            label = label.replace("'", "").replace('"', "")
            if label not in self.classes_:  # try again
                label = random.choices(self.classes_, self.probabilities_)[0]
        return label

    def fit(
        self,
        X: Optional[Union[np.ndarray, pd.Series, List[str]]],
        y: Union[np.ndarray, pd.Series, List[str]],
    ):
        y = self._to_np(y)
        return super().fit(X, y)


class MultiLabelZeroShotGPTClassifier(_BaseZeroShotGPTClassifier):
    def __init__(
        self,
        openai_key: Optional[str] = None,
        openai_org: Optional[str] = None,
        openai_model: str = "gpt-3.5-turbo",
        max_labels: int = 3,
    ):
        super().__init__(openai_key, openai_org, openai_model)
        if max_labels < 2:
            raise ValueError("max_labels should be at least 2")
        self.max_labels = max_labels

    def _extract_labels(self, y) -> List[str]:
        labels = []
        for l in y:
            for j in l:
                labels.append(j)

        return labels

    def _get_prompt(self, x) -> str:
        return build_zero_shot_prompt_mlc(x, repr(self.classes_), self.max_labels)

    def _predict_single(self, x):
        completion = self._get_chat_completion(x)
        try:
            labels = extract_json_key(completion.choices[0].message["content"], "label")
            if not isinstance(labels, list):
                raise RuntimeError("Invalid labels type, expected list")
        except Exception:
            labels = []

        labels = list(filter(lambda l: l in self.classes_, labels))

        if len(labels) > self.max_labels:
            labels = labels[: self.max_labels - 1]
        elif len(labels) < 1:
            labels = [random.choices(self.classes_, self.probabilities_)[0]]
        return labels

    def fit(
        self,
        X: Optional[Union[np.ndarray, pd.Series, List[str]]],
        y: List[List[str]],
    ):
        return super().fit(X, y)



import json
from time import sleep

import openai

from skllm.openai.credentials import set_credentials
from skllm.utils import find_json_in_string


def construct_message(role, content):
    if role not in ("system", "user", "assistant"):
        raise ValueError("Invalid role")
    return {"role": role, "content": content}


def get_chat_completion(messages, key, org, model="gpt-3.5-turbo", max_retries=3):
    set_credentials(key, org)
    error_msg = None
    error_type = None
    for _ in range(max_retries):
        try:
            completion = openai.ChatCompletion.create(
                model=model, temperature=0.0, messages=messages
            )
            return completion
        except Exception as e:
            error_msg = str(e)
            error_type = type(e).__name__
            sleep(3)
    print(
        f"Could not obtain the completion after {max_retries} retries: `{error_type} ::"
        f" {error_msg}`"
    )


def extract_json_key(json_, key):
    try:
        json_ = json_.replace("\n", "")
        json_ = find_json_in_string(json_)
        as_json = json.loads(json_)
        if key not in as_json.keys():
            raise KeyError("The required key was not found")
        return as_json[key]
    except Exception:
        return None



import numpy as np
from numpy import ndarray
import pandas as pd
from skllm.utils import to_numpy as _to_numpy
from typing import Any, Optional, Union, List
from skllm.openai.mixin import OpenAIMixin as _OAIMixin
from tqdm import tqdm
from sklearn.base import (
    BaseEstimator as _BaseEstimator,
    TransformerMixin as _TransformerMixin,
)
from skllm.openai.chatgpt import (
    construct_message,
    get_chat_completion,
)

class BaseZeroShotGPTTransformer(_BaseEstimator, _TransformerMixin, _OAIMixin): 
    
    system_msg = "You are an scikit-learn transformer."
    default_output = "Output is unavailable"

    def _get_chat_completion(self, X):
        prompt = self._get_prompt(X)
        msgs = []
        msgs.append(construct_message("system", self.system_msg))
        msgs.append(construct_message("user", prompt))
        completion = get_chat_completion(
            msgs, self._get_openai_key(), self._get_openai_org(), self.openai_model
        )
        try:
            return completion.choices[0].message["content"]
        except Exception as e:
            print(f"Skipping a sample due to the following error: {str(e)}")
            return self.default_output
            
    def fit(self, X: Any = None, y: Any = None, **kwargs: Any):
        return self

    def transform(self, X: Optional[Union[np.ndarray, pd.Series, List[str]]], **kwargs: Any) -> ndarray:
        X = _to_numpy(X)
        transformed = []
        for i in tqdm(range(len(X))):
            transformed.append(
                self._get_chat_completion(X[i])
            )
        transformed = np.asarray(transformed, dtype = object)
        return transformed

    def fit_transform(self, X: Optional[Union[np.ndarray, pd.Series, List[str]]], y=None, **fit_params) -> ndarray:
        return self.fit(X, y).transform(X)


from typing import Union

from skllm.prompts.templates import (
    FEW_SHOT_CLF_PROMPT_TEMPLATE,
    SUMMARY_PROMPT_TEMPLATE,
    TRANSLATION_PROMPT_TEMPLATE,
    ZERO_SHOT_CLF_PROMPT_TEMPLATE,
    ZERO_SHOT_MLCLF_PROMPT_TEMPLATE,
)

# TODO add validators


def build_zero_shot_prompt_slc(
    x: str, labels: str, template: str = ZERO_SHOT_CLF_PROMPT_TEMPLATE
) -> str:
    """Builds a prompt for zero-shot single-label classification.

    Parameters
    ----------
    x : str
        sample to classify
    labels : str
        candidate labels in a list-like representation
    template : str
        prompt template to use, must contain placeholders for all variables, by default ZERO_SHOT_CLF_PROMPT_TEMPLATE

    Returns
    -------
    str
        prepared prompt
    """
    return template.format(x=x, labels=labels)


def build_few_shot_prompt_slc(
    x: str,
    labels: str,
    training_data: str,
    template: str = FEW_SHOT_CLF_PROMPT_TEMPLATE,
) -> str:
    """Builds a prompt for zero-shot single-label classification.

    Parameters
    ----------
    x : str
        sample to classify
    labels : str
        candidate labels in a list-like representation
    training_data : str
        training data to be used for few-shot learning
    template : str
        prompt template to use, must contain placeholders for all variables, by default ZERO_SHOT_CLF_PROMPT_TEMPLATE

    Returns
    -------
    str
        prepared prompt
    """
    return template.format(x=x, labels=labels, training_data=training_data)


def build_zero_shot_prompt_mlc(
    x: str,
    labels: str,
    max_cats: Union[int, str],
    template: str = ZERO_SHOT_MLCLF_PROMPT_TEMPLATE,
) -> str:
    """Builds a prompt for zero-shot multi-label classification.

    Parameters
    ----------
    x : str
        sample to classify
    labels : str
        candidate labels in a list-like representation
    max_cats : Union[int,str]
        maximum number of categories to assign
    template : str
        prompt template to use, must contain placeholders for all variables, by default ZERO_SHOT_MLCLF_PROMPT_TEMPLATE

    Returns
    -------
    str
        prepared prompt
    """
    return template.format(x=x, labels=labels, max_cats=max_cats)


def build_summary_prompt(
    x: str, max_words: Union[int, str], template: str = SUMMARY_PROMPT_TEMPLATE
) -> str:
    """Builds a prompt for text summarization.

    Parameters
    ----------
    x : str
        sample to summarize
    max_words : Union[int,str]
        maximum number of words to use in the summary
    template : str
        prompt template to use, must contain placeholders for all variables, by default SUMMARY_PROMPT_TEMPLATE

    Returns
    -------
    str
        prepared prompt
    """
    return template.format(x=x, max_words=max_words)


def build_translation_prompt(
    x: str, output_language: str, template: str = TRANSLATION_PROMPT_TEMPLATE
) -> str:
    """Builds a prompt for text translation.

    Parameters
    ----------
    x : str
        sample to translate
    output_language : str
        language to translate to
    template : str
        prompt template to use, must contain placeholders for all variables, by default TRANSLATION_PROMPT_TEMPLATE

    Returns
    -------
    str
        prepared prompt
    """
    return template.format(x=x, output_language=output_language)



from sklearn.base import (
    BaseEstimator as _BaseEstimator,
    TransformerMixin as _TransformerMixin,
)
from typing import Any, Optional, Union, List
from tqdm import tqdm
import numpy as np
from numpy import ndarray
import pandas as pd
from skllm.openai.mixin import OpenAIMixin as _OAIMixin
from skllm.openai.embeddings import get_embedding as _get_embedding
from skllm.utils import to_numpy as _to_numpy


class GPTVectorizer(_BaseEstimator, _TransformerMixin, _OAIMixin):
    def __init__(
        self,
        openai_embedding_model: str = "text-embedding-ada-002",
        openai_key: Optional[str] = None,
        openai_org: Optional[str] = None,
    ):
        self.openai_embedding_model = openai_embedding_model
        self._set_keys(openai_key, openai_org)

    def fit(self, X: Any = None, y: Any = None, **kwargs):
        return self

    def transform(self, X: Optional[Union[np.ndarray, pd.Series, List[str]]]) -> ndarray:
        X = _to_numpy(X)
        embeddings = []
        for i in tqdm(range(len(X))):
            embeddings.append(
                _get_embedding(X[i], self._get_openai_key(), self._get_openai_org())
            )
        embeddings = np.asarray(embeddings)
        return embeddings

    def fit_transform(self, X: Optional[Union[np.ndarray, pd.Series, List[str]]], y=None, **fit_params) -> ndarray:
        return self.fit(X, y).transform(X)



from typing import Optional
from skllm.config import SKLLMConfig as _Config

class OpenAIMixin:

    def _set_keys(self, key: Optional[str] = None, org: Optional[str] = None) -> None:
        self.openai_key = key
        self.openai_org = org

    def _get_openai_key(self) -> str:
        key = self.openai_key
        if key is None:
            key = _Config.get_openai_key()
        if key is None:
            raise RuntimeError("OpenAI key was not found")
        return key

    def _get_openai_org(self) -> str:
        key = self.openai_org
        if key is None:
            key = _Config.get_openai_org()
        if key is None:
            raise RuntimeError("OpenAI organization was not found")
        return key
 


from typing import Optional

from skllm.openai.base_gpt import BaseZeroShotGPTTransformer as _BaseGPT
from skllm.prompts.builders import build_summary_prompt


class GPTSummarizer(_BaseGPT):
    system_msg = "You are a text summarizer."
    default_output = "Summary is unavailable."

    def __init__(
        self,
        openai_key: Optional[str] = None,
        openai_org: Optional[str] = None,
        openai_model: str = "gpt-3.5-turbo",
        max_words: int = 15,
    ):
        self._set_keys(openai_key, openai_org)
        self.openai_model = openai_model
        self.max_words = max_words

    def _get_prompt(self, X) -> str:
        return build_summary_prompt(X, self.max_words)



import numpy as np
import pandas as pd


def to_numpy(X):
    if isinstance(X, pd.Series):
        X = X.to_numpy().astype(object)
    elif isinstance(X, list):
        X = np.asarray(X, dtype=object)
    if isinstance(X, np.ndarray) and len(X.shape) > 1:
        X = np.squeeze(X)
    return X


def find_json_in_string(string):
    start = string.find("{")
    end = string.rfind("}")
    if start != -1 and end != -1:
        json_string = string[start : end + 1]
    else:
        json_string = {}
    return json_string



import unittest
import json
from unittest.mock import patch, MagicMock
import numpy as np
from skllm.models.gpt_zero_shot_clf import ZeroShotGPTClassifier, MultiLabelZeroShotGPTClassifier

class TestZeroShotGPTClassifier(unittest.TestCase):

    @patch("skllm.models.gpt_zero_shot_clf.get_chat_completion", return_value=MagicMock())
    def test_fit_predict(self, mock_get_chat_completion):
        clf = ZeroShotGPTClassifier(openai_key="mock_key", openai_org="mock_org")  # Mock keys
        X = np.array(["text1", "text2", "text3"])
        y = np.array(["class1", "class2", "class1"])
        clf.fit(X, y)

        self.assertEqual(set(clf.classes_), set(["class1", "class2"]))
        self.assertEqual(clf.probabilities_, [2/3, 1/3])

        mock_get_chat_completion.return_value.choices[0].message = {"content": json.dumps({"label": "class1"})}
        predictions = clf.predict(X)

        self.assertEqual(predictions, ["class1", "class1", "class1"])

class TestMultiLabelZeroShotGPTClassifier(unittest.TestCase):

    @patch("skllm.models.gpt_zero_shot_clf.get_chat_completion", return_value=MagicMock())
    def test_fit_predict(self, mock_get_chat_completion):
        clf = MultiLabelZeroShotGPTClassifier(openai_key="mock_key", openai_org="mock_org")  # Mock keys
        X = np.array(["text1", "text2", "text3"])
        y = [["class1", "class2"], ["class1", "class2"], ["class1", "class2"]]  # Adjusted y to ensure [0.5, 0.5] probability
        clf.fit(X, y)

        self.assertEqual(set(clf.classes_), set(["class1", "class2"]))
        self.assertEqual(clf.probabilities_, [0.5, 0.5])

        mock_get_chat_completion.return_value.choices[0].message = {"content": json.dumps({"label": ["class1", "class2"]})}
        predictions = clf.predict(X)

        self.assertEqual(predictions, [["class1", "class2"], ["class1", "class2"], ["class1", "class2"]])

if __name__ == '__main__':
    unittest.main()



from typing import Optional

from skllm.openai.base_gpt import BaseZeroShotGPTTransformer as _BaseGPT
from skllm.prompts.builders import build_translation_prompt


class GPTTranslator(_BaseGPT):
    """A text translator."""

    system_msg = "You are a text translator."
    default_output = "Translation is unavailable."

    def __init__(
        self,
        openai_key: Optional[str] = None,
        openai_org: Optional[str] = None,
        openai_model: str = "gpt-3.5-turbo",
        output_language: str = "English",
    ):
        self._set_keys(openai_key, openai_org)
        self.openai_model = openai_model
        self.output_language = output_language

    def _get_prompt(self, X: str) -> str:
        """Generates the prompt for the given input.

        Parameters
        ----------
        X : str
            sample to translate

        Returns
        -------
        str
            translated sample
        """
        return build_translation_prompt(X, self.output_language)
